package org.joget.tutorial;

import java.util.ArrayList;
import java.util.Collection;
import org.joget.apps.app.dao.UserviewDefinitionDao;
import org.joget.apps.app.model.AppDefinition;
import org.joget.apps.app.model.UserviewDefinition;
import org.joget.apps.app.service.AppPluginUtil;
import org.joget.apps.app.service.AppService;
import org.joget.apps.app.service.AppUtil;
import org.joget.apps.generator.model.GeneratorPlugin;
import org.joget.apps.generator.model.GeneratorResult;
import org.joget.apps.generator.service.GeneratorUtil;
import org.joget.commons.util.LogUtil;
import org.joget.commons.util.StringUtil;
import org.joget.commons.util.UuidGenerator;
import org.joget.workflow.util.WorkflowUtil;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

public class FormSubmissionStatisticsGenerator extends GeneratorPlugin {
    
    private final static String MESSAGE_PATH = "messages/FormSubmissionStatisticsGenerator";
    
    public String getName() {
        return "File Link Datalist Formatter";
    }
 
    public String getVersion() {
        return "5.0.0";
    }
    
    public String getClassName() {
        return getClass().getName();
    }
 
    public String getLabel() {
        //support i18n
        return AppPluginUtil.getMessage("org.joget.tutorial.FormSubmissionStatisticsGenerator.pluginLabel", getClassName(), MESSAGE_PATH);
    }
    
    public String getDescription() {
        //support i18n
        return AppPluginUtil.getMessage("org.joget.tutorial.FormSubmissionStatisticsGenerator.pluginDesc", getClassName(), MESSAGE_PATH);
    }
 
    public String getPropertyOptions() {
        String options = AppUtil.readPluginResource(getClassName(), "/properties/formSubmissionStatisticsGenerator.json", null, true, MESSAGE_PATH);
        
        //populate value like [formName]
        options = GeneratorUtil.populateFormMeta(options, getFormId(), getAppDefinition());
        
        //populate value of [default_userviewId]
        options = options.replace("[default_userviewId]", GeneratorUtil.getFirstAvailableUserviewId(getAppDefinition()));
        
        return options;
    }

    @Override
    public String getExplanation() {
        //support i18n
        return AppPluginUtil.getMessage("generator.formSubmissionStatistics.explanation", getClassName(), MESSAGE_PATH);
    }

    @Override
    public GeneratorResult generate() {
        GeneratorResult result = new GeneratorResult();
        AppDefinition appDef = getAppDefinition();
        try {
            
            //get userview
            UserviewDefinitionDao userviewDefinitionDao = (UserviewDefinitionDao) AppUtil.getApplicationContext().getBean("userviewDefinitionDao");
            UserviewDefinition userviewDef = null;
            String json = null;
            String userviewId = getPropertyString("userviewId");
            String userviewName;
            String userviewDesc;
            if (userviewId != null && !userviewId.isEmpty()) {         
                userviewDef = userviewDefinitionDao.loadById(userviewId, appDef);
            }
            if (userviewDef != null) {
                userviewName = userviewDef.getName();
                userviewDesc = userviewDef.getDescription();
                json = userviewDef.getJson();
            } else {
                userviewId = getPropertyString("userviewNewId");
                int count = 0;
                while (isExist(userviewId, userviewDefinitionDao)) {
                    count++;
                    userviewId = userviewId + count;
                }
                userviewName = getPropertyString("userviewName");
                userviewDesc = getPropertyString("userviewDesc");
            }
            if (json == null || json.isEmpty()) {
                //create a new userview json
                json = GeneratorUtil.createNewUserviewJson(userviewId, userviewName, userviewDesc);
            }
            
            //add the category json to userview json
            json = GeneratorUtil.addCategoryJsonToUserviewJson(getCategoryJson(appDef), json);
            
            if (userviewDef != null) { 
                userviewDef.setJson(json);
                userviewDefinitionDao.update(userviewDef);
            }else {
                userviewDef = new UserviewDefinition();
                userviewDef.setJson(json);
                userviewDef.setId(userviewId);
                userviewDef.setName(userviewName);
                userviewDef.setAppDefinition(appDef);
                userviewDefinitionDao.add(userviewDef);
                
                //Set current published version
                final AppDefinition currentAppDef = appDef;
                TransactionTemplate transactionTemplate = (TransactionTemplate)AppUtil.getApplicationContext().getBean("transactionTemplate");
                transactionTemplate.execute(new TransactionCallback<Object>() {
                    public Object doInTransaction(TransactionStatus ts) {
                        AppService appService = (AppService)AppUtil.getApplicationContext().getBean("appService");
                        appService.publishApp(currentAppDef.getId(), currentAppDef.getVersion().toString());
                        return null;
                    }
                });
            }
            
            String editLink = WorkflowUtil.getHttpServletRequest().getContextPath() + "/web/console/app/"+appDef.getAppId()+"/"+appDef.getVersion()+"/userview/builder/"+userviewId;
            String msg = AppPluginUtil.getMessage("generator.formSubmissionStatistics.msg.success", getClassName(), MESSAGE_PATH);
            msg = msg.replace("[url]", editLink);
            
            result.setMessage(msg);
        } catch (Exception e) {
            result.setError(true);
            result.setMessage(AppPluginUtil.getMessage("generator.formSubmissionStatistics.msg.error", getClassName(), MESSAGE_PATH));
            LogUtil.error(getClassName(), e, "Not able to generate the menus");
        }
        
        return result;
    }
    
    /**
     * Retrieves the category JSON definition 
     * @param appDef
     * @return 
     */
    protected String getCategoryJson(AppDefinition appDef) {
        Collection<String> args = new ArrayList<String>();
        
        AppService appService = (AppService)AppUtil.getApplicationContext().getBean("appService");
        String tabelName = appService.getFormTableName(appDef, getFormId());
        
        args.add(UuidGenerator.getInstance().getUuid()); //category id
        args.add(StringUtil.escapeString(getPropertyString("categoryLabel"), StringUtil.TYPE_JSON, null)); //category label
        args.add(UuidGenerator.getInstance().getUuid()); //monthly menu uuid
        args.add(StringUtil.escapeString(getPropertyString("monthlyMenuId"), StringUtil.TYPE_JSON, null)); //monthly menu custom id
        args.add(StringUtil.escapeString(getPropertyString("monthlyMenuLabel"), StringUtil.TYPE_JSON, null)); //monthly menu label
        args.add(StringUtil.escapeString(getPropertyString("monthlyChartTitle"), StringUtil.TYPE_JSON, null)); //monthly chart title
        args.add(StringUtil.escapeString(getPropertyString("monthlyXAxisLabel"), StringUtil.TYPE_JSON, null)); //monthly x-axis label
        args.add(StringUtil.escapeString(getPropertyString("yAxisLabel"), StringUtil.TYPE_JSON, null)); //monthly y-axis label
        args.add(StringUtil.escapeString(tabelName, StringUtil.TYPE_JSON, null)); //monthly form table name
        args.add(StringUtil.escapeString(getPropertyString("yearLabel"), StringUtil.TYPE_JSON, null)); //monthly year label
        args.add(StringUtil.escapeString(getPropertyString("showLabel"), StringUtil.TYPE_JSON, null)); //monthly show button label
        args.add(UuidGenerator.getInstance().getUuid()); //daily menu uuid
        args.add(StringUtil.escapeString(getPropertyString("dailyMenuId"), StringUtil.TYPE_JSON, null)); //daily menu custom id
        args.add(StringUtil.escapeString(getPropertyString("dailyMenuLabel"), StringUtil.TYPE_JSON, null)); //daily menu label
        args.add(StringUtil.escapeString(getPropertyString("dailyChartTitle"), StringUtil.TYPE_JSON, null)); //daily chart title
        args.add(StringUtil.escapeString(getPropertyString("dailyXAxisLabel"), StringUtil.TYPE_JSON, null)); //daily x-axis label
        args.add(StringUtil.escapeString(getPropertyString("yAxisLabel"), StringUtil.TYPE_JSON, null)); //daily y-axis label
        args.add(StringUtil.escapeString(tabelName, StringUtil.TYPE_JSON, null)); //daily form table name
        args.add(StringUtil.escapeString(getPropertyString("yearLabel"), StringUtil.TYPE_JSON, null)); //daily year label
        args.add(StringUtil.escapeString(getPropertyString("monthLabel"), StringUtil.TYPE_JSON, null)); //daily month label
        args.add(StringUtil.escapeString(getPropertyString("showLabel"), StringUtil.TYPE_JSON, null)); //daily show button label
        
        String json = AppUtil.readPluginResource(getClass().getName(), "/resources/category.json", args.toArray(), true, null);
        
        return json;
    }
    
    /**
     * Checks for a userview is already exist
     * @param id
     * @param userviewDefinitionDao
     * @return 
     */
    protected boolean isExist(String id, UserviewDefinitionDao userviewDefinitionDao) {
        Long count = userviewDefinitionDao.count("AND id = ?", new String[]{id}, getAppDefinition());
        return count > 0;
    }
}
